from src.nfixplanet.annotate.processing import filter_and_write_files

hmm_file = "/g/scb2/bork/ustick/nfixplanet_pipeline/01_annotation_pipeline/data/nfixplanet_test_full/02_test_hmm.tbl"

filter_and_write_files(hmm_file, "tmp_full", 10)