# diff -s <(zgrep '^>' /g/scb/bork/robbani/lucas/nfix_1/scratch/32/c9f25e4f08d6a401ab84b30bde2b21/processed_fastqs/R1.fq.gz | sort) <(zgrep '^>' tmp/processed_fastqs/R1.fq.gz | sort) | head 
# diff -s <(zgrep '^>' /g/scb/bork/robbani/lucas/nfix_1/scratch/32/c9f25e4f08d6a401ab84b30bde2b21/processed_fastqs/R2.fq.gz | sort) <(zgrep '^>' tmp/processed_fastqs/R2.fq.gz | sort) | head 
# diff -s <(zgrep '^>' /g/scb/bork/robbani/lucas/nfix_1/scratch/32/c9f25e4f08d6a401ab84b30bde2b21/processed_fastqs/RS.fq.gz | sort) <(zgrep '^>' tmp/processed_fastqs/RS.fq.gz | sort) | head 

# diff -s <(zgrep '^>' /g/scb/bork/robbani/lucas/nfix_1/scratch/d8/277df3a95f34aee7722c98a88b059f/cleaned_fastqs/R1.clean_1.fastq.gz | sort) <(zgrep '^>' tmp/cleaned_fastqs/R1.clean_1.fastq.gz | sort) | head 
# diff -s <(zgrep '^>' /g/scb/bork/robbani/lucas/nfix_1/scratch/d8/277df3a95f34aee7722c98a88b059f/cleaned_fastqs/R2.clean_2.fastq.gz | sort) <(zgrep '^>' tmp/cleaned_fastqs/R2.clean_2.fastq.gz | sort) | head 
diff -s <(zgrep '^>' /g/scb/bork/robbani/lucas/nfix_1/scratch/d8/277df3a95f34aee7722c98a88b059f/cleaned_fastqs/RS.clean.fastq.gz | sort) <(zgrep '^>' tmp/cleaned_fastqs/RS.clean.fastq.gz | sort) | head 