# diff -u tests/references/output/map_pipeline/SAMN06117828_sample_coverage.tsv /g/scb/bork/robbani/lucas/nfix_results/SAMN06117828_sample_coverage.tsv | delta --side-by-side
# diff -u tmp/SAMN06117828_sample_coverage.tsv /g/scb/bork/robbani/lucas/nfix_results/SAMN06117828_sample_coverage.tsv | delta --side-by-side

# diff -u -s tmp/SAMN06117828_coverage.tsv tests/references/output/map_pipeline/SAMN06117828_coverage.tsv | delta --side-by-side


# diff -u -s <(tail +2 tmp_full/vnfD.tsv | sort | head) <(sort /g/scb2/bork/ustick/nfixplanet_pipeline/01_annotation_pipeline/data/nfixplanet_test_full/QC_sequences/04_vnfD_final.out | head) | delta --side-by-side

FOLDER1="tmp_full"
FOLDER2="/g/scb/bork/robbani/nfixplanet/annotate_results/"

files=$(ls "$FOLDER1")

for file in $files; do
    echo "--- Diffing: $file ---"
    diff "$FOLDER1/$file" "$FOLDER2/$file"
    echo ""
done