in_orfs=/g/scb2/bork/ustick/nfixplanet_pipeline/01_annotation_pipeline/data/nfixplanet_proto_db/sequences/protoDB_01_all_gene_seqs.fna
in_hmms=/g/scb2/bork/ustick/nfixplanet_pipeline/01_annotation_pipeline/data/nfixplanet_test_full/02_test_hmm.tbl
out_path=tmp_med

nfixplanet annotate \
  --input_hmms "$in_hmms" \
  --output_directory "$out_path" \
  --genomic_context_range 10 \
  --cpus 4 \
  --verbose